"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Migration = void 0;
const child_process_1 = __importDefault(require("child_process"));
const path_1 = __importStar(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const utils_1 = require("@ezez/utils");
const semver_1 = __importDefault(require("semver"));
const dirname_js_1 = require("./dirname/dirname.js");
const thisDir = path_1.default.dirname(path_1.default.dirname(dirname_js_1.dirname));
const run = async (command, args, options) => {
    return new Promise((resolve, reject) => {
        const proc = child_process_1.default.spawn(command, args, options);
        proc.stdout.on("data", (data) => {
            console.info(String(data).trim());
        });
        proc.stderr.on("data", (data) => {
            console.info(String(data).trim());
        });
        proc.on("error", (error) => {
            reject(error);
        });
        proc.on("close", (code) => {
            if (!code) {
                resolve();
                return;
            }
            reject(new Error(`${command} ${args.join(" ")} exited with code ${code}`));
        });
    });
};
const PKG_JSON_INDENT = 2;
const JSON_INDENT = 4;
class Migration {
    constructor(opts) {
        this._targetDir = opts.targetDir;
        this._pkg = opts.pkg;
    }
    _savePkg() {
        return fs_extra_1.default.writeFile((0, path_1.join)(this._targetDir, "package.json"), JSON.stringify(this._pkg, null, PKG_JSON_INDENT));
    }
    get jsx() {
        return Boolean(this._pkg.libraryTemplate.jsx);
    }
    get targetDir() {
        return this._targetDir;
    }
    get pkg() {
        return this._pkg;
    }
    async updatePkg(fn) {
        await fn(this._pkg);
        return this._savePkg();
    }
    async setPath(objPath, value) {
        (0, utils_1.set)(this._pkg, objPath, value);
        await this._savePkg();
    }
    async setScript(scriptName, value) {
        await this.setPath(["scripts", scriptName], value);
    }
    async deletePath(objPath) {
        const thePath = typeof objPath === "string" ? objPath.split(".") : [...objPath];
        const last = thePath.pop();
        const item = (0, utils_1.get)(this._pkg, thePath);
        if (item == null || !last) {
            return;
        }
        delete item[last];
        await this._savePkg();
    }
    async deleteScript(scriptName) {
        await this.deletePath(["scripts", scriptName]);
    }
    async upgradeScript(scriptName, oldValue, newValue, skipIfOldValueDifferent = false) {
        try {
            this.assertScript(scriptName, oldValue, new Error("cannot update " + scriptName + " script as it was modified\n  "
                + "wanted new value:\n  `" + newValue + "`"));
        }
        catch (e) {
            if (skipIfOldValueDifferent) {
                return;
            }
            throw e;
        }
        await this.setScript(scriptName, newValue);
    }
    async addDependency(name, version) {
        await this.setPath(["dependencies", name], version);
    }
    async addDevDependency(name, version) {
        await this.setPath(["devDependencies", name], version);
    }
    async upgradeDependency(name, version, defaultType = "dependencies") {
        var _a;
        const type = (_a = this.findDependency(name)) !== null && _a !== void 0 ? _a : defaultType;
        await this.setPath([type, name], version);
    }
    async safelyUpgradeDependency(name, version) {
        var _a;
        const type = this.findDependency(name);
        if (!type) {
            throw new Error(`Cannot upgrade ${name}, because it is not installed`);
        }
        const depVersion = (_a = this._pkg[type]) === null || _a === void 0 ? void 0 : _a[name];
        const minVersion = semver_1.default.minVersion(depVersion);
        if (!minVersion) {
            throw new TypeError(`Cannot parse current version of ${name}`);
        }
        const minNewVersion = semver_1.default.minVersion(version);
        if (!minNewVersion) {
            throw new TypeError(`Cannot parse next version of ${name}`);
        }
        if (!semver_1.default.gt(minNewVersion, minVersion)) {
            throw new Error(`Currently installed version of ${name} ${minVersion.version} is higher than ${minNewVersion.version}`);
        }
        await this.setPath([type, name], version);
    }
    findDependency(name) {
        var _a, _b;
        const deps = (_a = this._pkg.dependencies) !== null && _a !== void 0 ? _a : {};
        const dev = (_b = this._pkg.devDependencies) !== null && _b !== void 0 ? _b : {};
        if (deps[name]) {
            return "dependencies";
        }
        if (dev[name]) {
            return "devDependencies";
        }
        return null;
    }
    async sortPackageJson() {
        await this.updatePkg((pkg) => {
            var _a, _b;
            pkg.dependencies = (0, utils_1.sortProps)((_a = pkg.dependencies) !== null && _a !== void 0 ? _a : {});
            pkg.devDependencies = (0, utils_1.sortProps)((_b = pkg.devDependencies) !== null && _b !== void 0 ? _b : {});
            return pkg;
        });
    }
    async sortTSConfigCompilerOptions(configFilePath) {
        await this.updateContentsJSON(configFilePath, (data) => {
            var _a;
            data.compilerOptions = (0, utils_1.sortProps)((_a = data.compilerOptions) !== null && _a !== void 0 ? _a : {});
            return data;
        });
    }
    async remove(pathName) {
        await fs_extra_1.default.remove((0, path_1.join)(this._targetDir, pathName));
    }
    async isDirEmpty(dirName) {
        const dir = (0, path_1.join)(this._targetDir, dirName);
        const files = await fs_extra_1.default.readdir(dir);
        return !files.filter(p => p !== "." && p !== "..").length;
    }
    async assertNoFile(fileName) {
        const target = (0, path_1.join)(this._targetDir, fileName);
        if (await fs_extra_1.default.pathExists(target)) {
            throw new Error(`File ${fileName} exists`);
        }
    }
    async assertFile(fileName) {
        const target = (0, path_1.join)(this._targetDir, fileName);
        if (!(await fs_extra_1.default.pathExists(target))) {
            throw new Error(`File ${fileName} does not exist`);
        }
    }
    async moveWithinLibrary(sourceName, targetName = null, overwrite = true) {
        await fs_extra_1.default.move((0, path_1.join)(this._targetDir, sourceName), (0, path_1.join)(this._targetDir, targetName !== null && targetName !== void 0 ? targetName : sourceName), { overwrite });
    }
    async copy(sourceName, targetName = null, overwrite = true) {
        await fs_extra_1.default.copy((0, path_1.join)(thisDir, sourceName), (0, path_1.join)(this._targetDir, targetName !== null && targetName !== void 0 ? targetName : sourceName), { overwrite });
    }
    async rename(source, target) {
        await fs_extra_1.default.rename((0, path_1.join)(this._targetDir, source), (0, path_1.join)(this._targetDir, target));
    }
    async updateContents(file, updater) {
        const target = (0, path_1.join)(this._targetDir, file);
        const data = String(await fs_extra_1.default.readFile(target));
        const contents = await updater(data);
        await fs_extra_1.default.writeFile(target, contents !== null && contents !== void 0 ? contents : "");
    }
    async pushLine(file, line) {
        const target = (0, path_1.join)(this._targetDir, file);
        const data = String(await fs_extra_1.default.readFile(target));
        const lines = data.split("\n");
        const hasLine = lines.map(s => s.trim()).find(s => s === line);
        if (hasLine) {
            return;
        }
        lines.push(line);
        await fs_extra_1.default.writeFile(target, lines.join("\n"));
    }
    async replaceLine(file, line, withLine) {
        const target = (0, path_1.join)(this._targetDir, file);
        const data = String(await fs_extra_1.default.readFile(target));
        const lines = data.split("\n");
        let replaced = false;
        const newLines = lines.map(s => {
            const trimmed = s.trim();
            if (trimmed === line) {
                replaced = true;
                return withLine;
            }
            return s;
        });
        if (!replaced) {
            return;
        }
        await fs_extra_1.default.writeFile(target, newLines.join("\n"));
    }
    async updateContentsJSON(file, updater) {
        const target = (0, path_1.join)(this._targetDir, file);
        const data = JSON.parse(String(await fs_extra_1.default.readFile(target)));
        const newData = await updater(data, utils_1.set.bind(null, data));
        await fs_extra_1.default.writeFile(target, JSON.stringify(newData !== null && newData !== void 0 ? newData : data, null, JSON_INDENT));
    }
    async updatePath(objPath, updater) {
        const current = (0, utils_1.get)(this._pkg, objPath);
        const newValue = await updater(String(current));
        await this.setPath(objPath, newValue);
    }
    assertPath(objPath, value, error) {
        if ((0, utils_1.get)(this._pkg, objPath) !== value) {
            throw error;
        }
    }
    assertNoPath(objPath, error) {
        if ((0, utils_1.get)(this._pkg, objPath) !== undefined) {
            throw error;
        }
    }
    assertScript(scriptName, value, error) {
        this.assertPath(["scripts", scriptName], value, error);
    }
    assertNoScript(scriptName, error) {
        this.assertNoPath(["scripts", scriptName], error);
    }
    assertNoDependency(name, error) {
        if (this._pkg.dependencies && name in this._pkg.dependencies) {
            throw error;
        }
    }
    assertNoDevDependency(name, error) {
        if (this._pkg.devDependencies && name in this._pkg.devDependencies) {
            throw error;
        }
    }
    assertNoAnyDependency(name, error) {
        this.assertNoDependency(name, error);
        this.assertNoDevDependency(name, error);
    }
    assertDevDependency(name, version, error) {
        if (!this._pkg.devDependencies
            || ((version && this._pkg.devDependencies[name] !== version)
                || (!version && !this._pkg.devDependencies[name]))) {
            throw error;
        }
    }
    async removeDependency(name) {
        await this.deletePath(["dependencies", name]);
    }
    async removeDevDependency(name) {
        await this.deletePath(["devDependencies", name]);
    }
    async removeAnyDependency(name) {
        const depType = this.findDependency(name);
        if (!depType) {
            return;
        }
        await this.deletePath([depType, name]);
    }
    yarn() {
        return this.run("yarn");
    }
    run(applicationPath, args = []) {
        return run(applicationPath, args, {
            cwd: this._targetDir,
            shell: true,
        });
    }
}
exports.Migration = Migration;
//# sourceMappingURL=Migration.js.map