import type { PackageJson } from "./types.js";
interface Data {
    [key: string]: unknown;
}
interface Options {
    targetDir: string;
    pkg: PackageJson;
}
type GetSetPath = string | string[];
type MaybePromise<T> = T | Promise<T>;
type ContentsUpdater = (fileContents: string) => (MaybePromise<string | undefined>);
type JSONContentsUpdater<Src = Data, Target = Src> = (fileData: Src, setFn: (objPath: GetSetPath, value: unknown) => void) => (Promise<Target | undefined> | Target | undefined);
type Dep = "dependencies" | "devDependencies";
declare class Migration {
    private readonly _targetDir;
    private readonly _pkg;
    constructor(opts: Options);
    private _savePkg;
    get jsx(): boolean;
    get targetDir(): string;
    get pkg(): PackageJson;
    updatePkg(fn: (pkg: PackageJson) => unknown): Promise<void>;
    setPath(objPath: GetSetPath, value: boolean | string | undefined | number | null): Promise<void>;
    setScript(scriptName: string, value: string | undefined): Promise<void>;
    deletePath(objPath: GetSetPath): Promise<void>;
    deleteScript(scriptName: string): Promise<void>;
    upgradeScript(scriptName: string, oldValue: string, newValue: string, skipIfOldValueDifferent?: boolean): Promise<void>;
    addDependency(name: string, version: string): Promise<void>;
    addDevDependency(name: string, version: string): Promise<void>;
    upgradeDependency(name: string, version: string, defaultType?: Dep): Promise<void>;
    safelyUpgradeDependency(name: string, version: string): Promise<void>;
    findDependency(name: string): "dependencies" | "devDependencies" | null;
    sortPackageJson(): Promise<void>;
    sortTSConfigCompilerOptions(configFilePath: string): Promise<void>;
    remove(pathName: string): Promise<void>;
    isDirEmpty(dirName: string): Promise<boolean>;
    assertNoFile(fileName: string): Promise<void>;
    assertFile(fileName: string): Promise<void>;
    moveWithinLibrary(sourceName: string, targetName?: string | null, overwrite?: boolean): Promise<void>;
    copy(sourceName: string, targetName?: string | null, overwrite?: boolean): Promise<void>;
    rename(source: string, target: string): Promise<void>;
    updateContents(file: string, updater: ContentsUpdater): Promise<void>;
    pushLine(file: string, line: string): Promise<void>;
    replaceLine(file: string, line: string, withLine: string): Promise<void>;
    updateContentsJSON<Src = Data, Target = Src>(file: string, updater: JSONContentsUpdater<Src, Target>): Promise<void>;
    updatePath(objPath: GetSetPath, updater: ContentsUpdater): Promise<void>;
    assertPath(objPath: GetSetPath, value: unknown, error: Error): void;
    assertNoPath(objPath: GetSetPath, error: Error): void;
    assertScript(scriptName: string, value: string, error: Error): void;
    assertNoScript(scriptName: string, error: Error): void;
    assertNoDependency(name: string, error: Error): void;
    assertNoDevDependency(name: string, error: Error): void;
    assertNoAnyDependency(name: string, error: Error): void;
    assertDevDependency(name: string, version: string | null, error: Error): void;
    removeDependency(name: string): Promise<void>;
    removeDevDependency(name: string): Promise<void>;
    removeAnyDependency(name: string): Promise<void>;
    yarn(): Promise<void>;
    run(applicationPath: string, args?: string[]): Promise<void>;
}
export { Migration };
export type { PackageJson };
//# sourceMappingURL=Migration.d.ts.map